
/**
 * Javier Abelln. 6 Abril 2006
 * 
 * Librera grfica
 */

package chuidiang.graficos;


import java.awt.geom.Rectangle2D;

import chuidiang.matematicas.CartesianasPolares;
import chuidiang.matematicas.UtilRectangulo;


/**
 * Zoom para un grfico polar.<br>
 *
 */
public class ZoomEscalaGraficaPolar
        implements InterfaceZoomable
{
   //~ Variables de instancia --------------------------------------------------

   /** Escala del grfico polar */
   private EscalaGraficaPolar escala;

   //~ Constructores -----------------------------------------------------------

   /**
    * Crea  un nuevo objeto de la clase ZoomEscalaGraficaPolar.<br>
    *
    * @param escala Escala del grfico polar.<br>
    */
   public ZoomEscalaGraficaPolar( EscalaGraficaPolar escala )
   {
      this.escala = escala;
   }

   //~ Metodos -----------------------------------------------------------------

   /**
    * Acerca el grfico polar el porcentaje indicado.<br>
    *
    * @param porcentaje en %.<br>
    */
   public void acercar( double porcentaje )
   {
      Rectangle2D extremos = this.escala.getExtremosCartesianos(  );
      Rectangle2D nuevo = UtilRectangulo.agrandar( 
            extremos,
            porcentaje );
      this.escala.tomaExtremosCartesianos( 
         nuevo.getX(  ),
         nuevo.getY(  ),
         nuevo.getX(  ) + nuevo.getWidth(  ),
         nuevo.getY(  ) + nuevo.getHeight(  ) );
   }

   /**
    * Aleja el grfico polar el porcentaje indicado.<br>
    *
    * @param porcentaje en %.<br>
    */
   public void alejar( double porcentaje )
   {
      Rectangle2D extremos = this.escala.getExtremosCartesianos(  );
      Rectangle2D nuevo = UtilRectangulo.reducir( 
            extremos,
            porcentaje );
      this.escala.tomaExtremosCartesianos( 
         nuevo.getX(  ),
         nuevo.getY(  ),
         nuevo.getX(  ) + nuevo.getWidth(  ),
         nuevo.getY(  ) + nuevo.getHeight(  ) );
   }

   /**
    * Desplaza el grfico polar de forma qun un punto dado pase a tener
    * unas nuevas coordenadas indicadas.<br>
    *
    * @param viejaX radio del punto.<br>
    * @param viejaY angulo en radianes.<br>
    * @param nuevaX nuevo radio deseado para el punto.<br>
    * @param nuevaY nuevo angulo en radianes deseado para el punto.<br>
    */
   public void desplazar( 
      double viejaX,
      double viejaY,
      double nuevaX,
      double nuevaY )
   {
      Rectangle2D extremos = escala.getExtremosCartesianos(  );
      double xn = CartesianasPolares.dameX( 
            nuevaX,
            nuevaY );
      double yn = CartesianasPolares.dameY( 
            nuevaX,
            nuevaY );
      double xv = CartesianasPolares.dameX( 
            viejaX,
            viejaY );
      double yv = CartesianasPolares.dameY( 
            viejaX,
            viejaY );

      escala.tomaExtremosCartesianos( 
         extremos.getMinX(  ) - ( xn - xv ),
         extremos.getMinY(  ) - ( yn - yv ),
         ( extremos.getMinX(  ) + extremos.getWidth(  ) ) - ( xn - xv ),
         ( extremos.getMinY(  ) + extremos.getHeight(  ) ) - ( yn - yv ) );
   }
}
